package com.link.core.kafka.dynamickafka;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import javax.annotation.Resource;
import org.apache.kafka.common.serialization.StringSerializer;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.kafka.core.DefaultKafkaProducerFactory;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.kafka.core.ProducerFactory;

@Configuration
public class DynamicKafkaProducerConfiguration {
   @Resource
   private DynamicKafkaProps dynamicKafkaProps;
   @Resource
   private ApplicationContext applicationContext;

   @Bean(
      name = {"dynamicKafkaTemplate"}
   )
   public Map<String, KafkaTemplate<String, String>> dynamicKafkaTemplate() {
      Map<String, KafkaTemplate<String, String>> dynamicKafkaMap = new HashMap(2);
      if (!this.dynamicKafkaProps.getEnable()) {
         return dynamicKafkaMap;
      } else {
         Map<String, ProducerFactory<String, String>> factoryMap = this.dynamicProducerFactory();
         Iterator<String> keyIter = factoryMap.keySet().iterator();

         while(keyIter.hasNext()) {
            String name = (String)keyIter.next();
            KafkaTemplate kafkaTemplate = new KafkaTemplate((ProducerFactory)factoryMap.get(name));
            dynamicKafkaMap.put(name, kafkaTemplate);
         }

         return dynamicKafkaMap;
      }
   }

   public Map<String, ProducerFactory<String, String>> dynamicProducerFactory() {
      Map<String, ProducerFactory<String, String>> producerFactoryMap = new HashMap(2);
      Map<String, Map<String, Object>> configs = this.dynamicProducerConfigs();
      Iterator<String> keyIter = configs.keySet().iterator();

      while(keyIter.hasNext()) {
         String name = (String)keyIter.next();
         ProducerFactory<String, String> factory = new DefaultKafkaProducerFactory((Map)configs.get(name));
         producerFactoryMap.put(name, factory);
      }

      return producerFactoryMap;
   }

   public Map<String, Map<String, Object>> dynamicProducerConfigs() {
      Map<String, Map<String, Object>> configMap = new HashMap(2);
      List<String> names = this.dynamicKafkaProps.getName();

      for(int i = 0; i < names.size(); ++i) {
         Map<String, Object> props = new HashMap();
         props.put("bootstrap.servers", this.dynamicKafkaProps.getProducerServers().get(i));
         props.put("retries", 0);
         props.put("acks", "0");
         props.put("max.block.ms", 60000);
         props.put("batch.size", 4096);
         props.put("linger.ms", 1);
         props.put("buffer.memory", 82428800);
         props.put("max.request.size", 52428800);
         props.put("max.message.bytes", 52428800);
         props.put("key.serializer", StringSerializer.class);
         props.put("value.serializer", StringSerializer.class);
         props.put("compression.type", "none");
         String name = (String)this.dynamicKafkaProps.getName().get(i);
         configMap.put(name, props);
      }

      return configMap;
   }
}
